<?php
/*
    phpPasswordProtector.php
    
    Copyright - Vincenzo Mennella 2015
    All right reserved
    
    This software is distribuited under the Envato Licence you've choosen.
    Please take a look at http://codecanyon.net/licenses/standard for licences details.
    
    Version: 1.1.0
    Last Update: 23/03/2015 
    
    Changelog:
        - 1.0.0 (15/03/2015)
            - First relase
            
        - 1.1.0 (23/03/2015)
            - Logs of logins
            - Remember me
            - Login with a sequence of keys (es "arrow up, arrow down, arrow down, arrow left, arrow right”)
*/

namespace phpPP;


class phpPasswordProtector 
{
    
    private function getUserConfig() 
    {
        /*
            Please edit just the code you find below this note!
            Everything you need to configure is marked with a comment "CONFIGURE ME" and a brief doc on how to set all the parameters in the right way!
            DO NOT TOUCH ANYTHING ELSE!
        */
        
        
        
        /*
            CONFIGURE ME! - STEP 1
            The variable $this->password_type allows you to set the encryption type of the passwords you'll set after
            just remove double slashes before the mode you want to use (remember to comment -add double slashes- the modes you won't use!!)
        
            EXAMPLE: set "text" if you'll write the passwords in plain text
            EXAMPLE: set "SHA1" if you'll write the passwords in SHA1. You can use any online service to generate an SHA1 string from plain text! (try http://www.timestampgenerator.com/tools/sha1-generator/) - this provides better secutiry!!
        
            The variable $this->phpPP_cllength allows you to set the length of the special combination lock (please look at step 2 configuration for more details). Use higher values for higher security. Minimum is 5. Set 0 to disable this feature.
            
        */
        $this->password_type = "text";  
        $this->phpPP_cllength = 5;  

        
                
        /*
            CONFIGURE ME! - STEP 2
            Set the username and passwords for your users. Remember: write the password respecting the mode you've set before (if you've choosen "text", write the password in plain text!)
            In order to add more users, simply create another line of code using the following example!
            
            NOTE: edit just the "insert_username_here" and "insert_password_here" fields, without deleting the quotes!
            NOTE: please provide UNIQUE usernames!
            
            
            !!SPECIAL COMBINATION LOCK GUIDE!!
            If you prefer, you can assign to each user a special "combination lock" password. This means that users can login using their special combination of keys (eg. arrow down, arrow down, arrow up, arrow right, arrow left) instead of their normal password.
            Please create a combination which is of the same length you decided before in $this->phpPP_cllength otherwise the login won't work!
            
            In order to assign the special combination, you have to put every part of the combination in a string (remember to respect the max length you set in $this->phpPP_cllength!!) separating each code with a minus sign (-). The codes for the combination are the following:
            au => "arrow up"
            ad => "arrow down"
            al => "arrow left"
            ar => "arrow right"
            (more codes will come in the next relases!)
            
            For example, if you want to create a code "arrow up, arrow down, arrow down, arrow left, arrow right" you must write "au-ad-ad-al-ar" in the "insert_combination_lock_here" field below
            
            Obviously, for security reasons, the user MUST write his username before using his special combination lock!
            !!
            
            
            EXAMPLE: $this->users["insert_username_here"] = array("insert_password_here", "insert_combination_lock_here");
        */
        $this->users["admin"] = array("admin");
        //$this->users["admin"] = array("$t_password", "au-ad-ad-al-ar");
        
        
        
        /*
            CONFIGURE ME! (optional) - STEP 3
            Set the access level for your users. With this setting on, you can grant or deny access to different pages to different users.
            NOTE: Please provide only numerical level
            NOTE: Higher levels grant higher access!
            NOTE: Users without a level can access EVERY page!
            
            NOTE: You must include $phpPasswordProtector->minimumLevelRequired(level_code) in your page in order to use this functionality in the pages. If you don't include this line of code, EVERY logged user can view the page!
            
            EXAMPLE: $this->levels["insert_username_here"] = 0;
        */
        $this->levels["demo2"] = 1;
        $this->levels["insert_username_here"] = 0;
        
        
        
        /*
            CONFIGURE ME! - STEP 4
            Set the path of your application and the path of the phpPasswordProtector Class.
            For example:
            If the application resides in your document root, write a single slash into this field. If your application resides in a subfolder (or a sub-subfolder) write the path of the folders up to your application.
            The phpPasswordProtector Class must reside inside your application. Set its path (relative to the applicationPath) inside $this->applicationClassPath; 
        */
        //$this->applicationPath = "/demos/envato/phpPasswordProtector/";
       // $this->applicationPath = "/clients2/master/landing/test/";
	   
        $this->applicationPath = "";
	   
        $this->applicationClassPath = "phpPasswordProtector/";
                
        
        
        /*
            CONFIGURE ME! - STEP 5
            Set the path of your success login redirect.
            This is the path where your user will be sent after a successfull login (relative to the "applicationPath" set above!)
        */
        $this->applicationPathSuccessLogin = "admin.php";
        
        
        
        /*
            CONFIGURE ME! - STEP 6
            Set the name of the page that will contain your HTML login form.
        */
        $this->applicationPathLoginForm = "login.php";
        
        
        
        /*
            CONFIGURE ME! (optional) - STEP 7
            Set the prefix of the session variable where the login informations of the user will be stored.
            You can leave this "as is" if you don't know what you're doing!
        */
        $this->phpSessVar = "phpPP_";
        
        
        
        /*
            CONFIGURE ME! (optional) - STEP 8
            You can store the logins of your users in a TXT file. Set true to enable this feature or false to disable it.
            
            Note that you must grant write permissions to the "log" folder inside the folder of phpPasswordProtector!
        */
        $this->saveLoginLogs = false;
        
        
        
        /*
            CONFIGURE ME! (optional) - STEP 9
            You can protect your app from bruteforcing attacks by blocking the login for the next hour for every user that enters a wrong password 5 times in a row. 
            
            Note that you must grant write permissions to the "log" folder inside the folder of phpPasswordProtector!
        */
        $this->bruteForceProtection = true;
        
        
        //DO NOT TOUCH ANYTHING BELOW THIS LINE!
        $ary_to_return = array(
            "password_type" => $this->password_type,
            "users" => $this->users,
            "levels" => $this->levels,
            "session_var" => $this->phpSessVar,
            "app_path" => $this->applicationPath,
            "application_class_path" => $this->applicationClassPath,
            "app_path_success_login" => $this->applicationPathSuccessLogin,
            "app_path_login_form" => $this->applicationPathLoginForm,
            "log_logins" => $this->saveLoginLogs,
            "brute_force_protection" => $this->bruteForceProtection
        );
        
        return $ary_to_return;
    }
    
    function __construct() 
    {
        session_start();
        $found_user = false;
        $this->phpPP_user = "";
        $this->phpPP_auth = "";
        
        $this->config = $this->getUserConfig();
        
        if (isset($_POST['phpPP_user'])) {
            $this->phpPP_user = $_POST['phpPP_user'];
            
            $previous_try = $this->manageUnsuccessfullLogin("get", $this->phpPP_user);  
            if ($previous_try['number_of_wrong'] > 5 && (time()-$previous_try['last_login_try']) <= 3600) {
                $this->doLogout('bruteforce_protection');
                die();
            }
        }
        
        if (isset($_COOKIE[$this->phpSessVar . "auth"]) && $_SESSION[$this->config['session_var'] . "_LOGIN_STATUS"] == "") {
            $found_user = $this->checkRememberCookie();
        }
        
        if (!$found_user) {
            if (!isset($_SESSION[$this->config['session_var'] . "_LOGIN_STATUS"])) {
                if ($this->phpPP_user != "" && $_POST['phpPP_pass'] != "") {
                    $this->doLogin();
                } else if ($this->phpPP_user != "" && $_POST['phpPP_pass'] == "" && $_POST['phpPP_clcontainer'] != "") {
                    $this->doLogin("combination_lock");
                } else {
                    if (isset($_GET['a'])) {
                        if ($_GET['a'] == "login") {
                            $this->doLogout('mandatory_fields');
                        }
                    } else {
                        if (basename($_SERVER['PHP_SELF']) != $this->config['app_path_login_form']) {
                            $this->doLogout('ask_relog');
                        }
                    }
                }
            } else {
                if (isset($_GET['a'])) {
                    if ($_GET['a'] == "logout") {
                        $this->doLogout('logout');
                    } else if ($_GET['a'] == "login") {
                        header("location:" . $this->config['app_path'] . $this->config['app_path_success_login']);
                    }
                }
            }
        }
            
    }

    private function createRememberCookie($user) 
    {
        setcookie($this->phpSessVar . "auth", sha1($user . $this->config['users'][$user][0]), time()+60*60*24*30, "/");
    }
    
    private function checkRememberCookie() 
    {
        $found_user = false;

        foreach($this->config['users'] as $user => $ary_passwords) {
            $sha1_string = sha1($user . $ary_passwords[0]);

            if ($_COOKIE[$this->phpSessVar . "auth"] == $sha1_string) {
                $this->phpPP_user = $user;
                $_POST['phpPP_pass'] = $ary_passwords[0];
                
                $found_user = true;
                $this->doLogin();
                
                break;
            }
        }

        return $found_user;
    }
    
    private function destroyRememberCookie() 
    {
        unset($_COOKIE[$this->phpSessVar . "auth"]);
        setcookie($this->phpSessVar . "auth", "", time()-3600, "/");
    }
    
    private function saveLoginLog($user) 
    {
        $loginsLogFile = fopen($this->config['application_class_path'] . "logs/logins.txt", "a");
        fwrite($loginsLogFile, "User: " . $user . ". Date and Time " . date("d/m/Y H:i:s") . ". \n");
        fclose($loginsLogFile);
    }
    
    private function doLogin($auth_type = "password") 
    {       
        $user = $this->phpPP_user;

        if ($auth_type == "password") {
            if ($this->config['password_type'] == "text") {
                $password = $_POST['phpPP_pass'];
            } else if ($this->config['password_type'] == "SHA1") {
                $password = sha1($_POST['phpPP_pass']);
            }
            
            $password_ary_index = 0;
        } else if ($auth_type == "combination_lock") {
            $password = substr($_POST['phpPP_clcontainer'], 0, -1);
            
            $password_ary_index = 1;
        }
        
                
        if (array_key_exists($user, $this->config['users'])) {
            if ($this->config['users'][$user][$password_ary_index] === $password) {
                $_SESSION[$this->config['session_var'] . "_LOGIN_STATUS"] = sha1($user);
                $_SESSION[$this->config['session_var'] . "_USERNAME"] = $user;
                $_SESSION[$this->config['session_var'] . "_USERLEVEL"] = $this->config['levels'][$user];
                
                if ($this->config['log_logins']) {
                    $this->saveLoginLog($user);
                }

                if ($_POST['phpPP_remember'] == "on") {
                    $this->createRememberCookie($user);
                }
                
                if ($this->config['brute_force_protection']) {
                    $this->manageUnsuccessfullLogin("remove", $user);
                }
        
                header("location:" . $this->config['app_path'] . $this->config['app_path_success_login']);
            } else {
                $this->doLogout('wrong_pass', $user);
            }
        } else {
            $this->doLogout('wrong_user');
        }
    }
    
    private function doLogout($par, $user = "") 
    {
        unset($_SESSION[$this->config['session_var'] . "_LOGIN_STATUS"]);
        session_destroy();
        $this->destroyRememberCookie();
        
        if ($par == "wrong_pass" && $this->config['brute_force_protection']) {
            $this->manageUnsuccessfullLogin("add", $user);
        }
        
        header("location:" . $this->config['app_path'] . $this->config['app_path_login_form'] . "?a=" . $par);
    }
    
    private function manageUnsuccessfullLogin($operation, $user) 
    {
        $unsuccessfullLoginsLogFile = fopen($this->config['application_class_path'] . "logs/unsuccessfull_logins.json", "a+");
        $prev_text = fread($unsuccessfullLoginsLogFile, filesize($this->config['application_class_path'] . "logs/unsuccessfull_logins.json"));
        $decoded_unsuccess = json_decode($prev_text, true);

        if ($operation == "add") {
            $number_of_wrong = $decoded_unsuccess[$user]["number_of_wrong"];
            
            $decoded_unsuccess[$user] = array(
                "number_of_wrong" => $number_of_wrong+1,
                "last_login_try" => time()
            );
            
        } else if ($operation == "remove") {
            unset($decoded_unsuccess[$user]);
        } else if ($operation == "get") {
            return $decoded_unsuccess[$user];
        }
        
        ftruncate($unsuccessfullLoginsLogFile, 0);
        fwrite($unsuccessfullLoginsLogFile, json_encode($decoded_unsuccess));
        
        fclose($unsuccessfullLoginsLogFile);
    }
    
    public function generateLogoutLink() 
    {
        return $this->config['app_path'] . $this->config['app_path_login_form'] . "?a=logout";
    }
    
    public function generateLoginLink() 
    {
        return $this->config['app_path'] . $this->config['app_path_login_form'] . "?a=login";
    }
    
    public function getLoggedUser() 
    {
        return $_SESSION[$this->config['session_var'] . "_USERNAME"];
    }
    
    public function minimumLevelRequired($level) 
    {
        if ($_SESSION[$this->config['session_var'] . "_USERLEVEL"] < $level && $_SESSION[$this->config['session_var'] . "_USERLEVEL"] != "") {
            $this->doLogout('low_level');
        }
    }
    
    public function getCombinationLockLength() 
    {
        return $this->phpPP_cllength;
    }
    
}

$phpPasswordProtector = new phpPasswordProtector();
